==> uhub-0.1.1-1956-src.tar.gz: uhub-0.1.1-1956/src/network.c <==
/*
 * uhub - A tiny ADC p2p connection hub
 * Copyright (C) 2007, Jan Vidar Krey
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 */

#include "uhub.h"


static int is_ipv6_supported = -1; /* -1 = CHECK, 0 = NO, 1 = YES */
static struct net_statistics stats;
static struct net_statistics stats_total;

int net_set_nonblocking(int fd, int toggle)
{
	int ret;
	uhub_log(log_trace, "net_set_nonblocking(): fd=%d", fd);
	ret = ioctl(fd, FIONBIO, &toggle);
    if (ret == -1)
	{
		uhub_log(log_error, "net_set_nonblocking(): ioctl failed (fd=%d): %s", fd, strerror(errno));
		return -1;
	}
	return 0;
}


int net_set_reuseaddress(int fd, int toggle)
{
	int ret;
	uhub_log(log_trace, "net_set_reuseaddress(): fd=%d", fd);
	ret = setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, &toggle, sizeof(toggle));
	if (ret == -1)
	{
		uhub_log(log_error, "net_set_reuseaddress(): setsockopt failed (fd=%d): %s", fd, strerror(errno));
		return -1;
	}
	return 0;
}


int net_close(int fd)
{
	int ret = close(fd);
	if (ret == 0)
	{
		net_stats_add_close();
	}
	else
	{
		if (fd != -1)
		{
			net_stats_add_error();
		}
	}
	return ret;
}


int net_accept(int fd)
{
	struct sockaddr_in6 addr;
	socklen_t addr_size;
	int ret = 0;
	addr_size = sizeof(struct sockaddr_in6);
	memset(&addr, 0, addr_size);
	uhub_log(log_trace, "net_accept() fd=%d", fd);
	ret = accept(fd, (struct sockaddr*) &addr, &addr_size);
	
	if (ret == -1)
	{
		switch (errno)
		{
#if defined(__linux__)
			case ENETDOWN:
			case EPROTO:
			case ENOPROTOOPT:
			case EHOSTDOWN:
			case ENONET:
			case EHOSTUNREACH:
			case EOPNOTSUPP:
				errno = EAGAIN;
#endif
			case EAGAIN:
				break;
			default:
				uhub_log(log_error, "net_accept(): accept failed (fd=%d, errno=%d, msg=%s)", fd, errno, strerror(errno));
				net_stats_add_error();
				return -1;
		}
	}
	else
	{
		net_stats_add_accept();
	}
	
	return ret;
}


int net_is_ipv6_supported()
{
	if (is_ipv6_supported == -1)
	{
		int ret = socket(AF_INET6, SOCK_DGRAM, IPPROTO_UDP);
		if (ret == -1)
		{
			if (errno == EAFNOSUPPORT)
			{
				is_ipv6_supported = 0;
				return 0;
			}
		
		}
		else
		{
			close(ret);
		}
		is_ipv6_supported = 1;
	}
	return is_ipv6_supported;
}


int net_socket_create(int af, int type, int protocol)
{
	int sd = socket(af, type, protocol);
	if (sd == -1)
	{
		uhub_log(log_error, "net_socket_create(): socket failed (errno=%d, msg=%s)", errno, strerror(errno));
	}
	return sd;
}


const char* net_get_peer_address(int fd)
{
	static char address[INET6_ADDRSTRLEN+1];
	struct sockaddr_in6 name;
	socklen_t namelen = sizeof(struct sockaddr_in6);
	int af = net_is_ipv6_supported() ? AF_INET6 : AF_INET;
	
	memset(address, 0, INET6_ADDRSTRLEN);
	
	if (getpeername(fd, (struct sockaddr*) &name, &namelen) != -1)
	{
		inet_ntop(af, (void*) &(&name)->sin6_addr, address, INET6_ADDRSTRLEN);
		if (strncmp(address, "::ffff:", 7) == 0) /* IPv6 mapped IPv4 address. */
		{
			return &address[7];
		}
		uhub_log(log_trace, "net_get_peer_address(): address=%s", address);
		return address;
	}
	else
	{
		uhub_log(log_error, "net_get_peer_address(): getsockname failed (fd=%d, errno=%d, msg=%s)", fd, errno, strerror(errno));
	}
	
	return "0.0.0.0";
}


ssize_t net_recv(int fd, void* buf, size_t len, int flags)
{
	ssize_t ret = recv(fd, buf, len, flags);
	if (ret >= 0)
	{
		net_stats_add_rx(ret);	
	}
	else
	{
		if (errno != EAGAIN)
			net_stats_add_error();
	}
	return ret;
}


ssize_t net_send(int fd, void* buf, size_t len, int flags)
{
	ssize_t ret = send(fd, buf, len, flags);
	if (ret >= 0)
	{
		net_stats_add_tx(ret);	
	}
	else
	{
		if (errno != EAGAIN)
			net_stats_add_error();
	}
	return ret;
}


void net_stats_initialize()
{
	memset(&stats_total, 0, sizeof(struct net_statistics));
	stats_total.timestamp = time(NULL);
	
	memset(&stats, 0, sizeof(struct net_statistics));
	stats.timestamp = time(NULL);
}


void net_stats_report()
{
	int factor = (time(NULL) - stats.timestamp);
	if (!factor) factor++;
	
	uhub_log(log_info, "Statistics  NET: tx=%d KB/s, rc=%d KB/s, accept=%d, closed=%d, errors=%d",
		(int) ((stats.tx / factor) / 1024),
		(int) ((stats.rx / factor) / 1024),
		(int) stats.accept,
		(int) stats.closed,
		(int) stats.errors);
}


void net_stats_reset()
{
	stats_total.tx += stats.tx;
	stats_total.rx += stats.rx;
	stats_total.accept += stats.accept;
	stats_total.errors += stats.errors;
	stats_total.closed += stats.closed;
	
	memset(&stats, 0, sizeof(struct net_statistics));
	stats.timestamp = time(NULL);
}


int net_stats_timeout()
{
	/* FIXME: Configurable time for dumping network statistics */
	return (time(NULL) - stats.timestamp > 60) ? 1 : 0;
}


void net_stats_add_tx(size_t bytes)
{
	stats.tx += bytes;
}


void net_stats_add_rx(size_t bytes)
{
	stats.rx += bytes;
}


void net_stats_add_accept()
{
	stats.accept++;
}


void net_stats_add_error()
{
	stats.errors++;
}


void net_stats_add_close()
{
	stats.closed++;
}

==> uhub-0.1.2-2020-src.tar.gz: uhub-0.1.2-2020/src/network.c <==
/*
 * uhub - A tiny ADC p2p connection hub
 * Copyright (C) 2007, Jan Vidar Krey
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 */

#include "uhub.h"


static int is_ipv6_supported = -1; /* -1 = CHECK, 0 = NO, 1 = YES */
static struct net_statistics stats;
static struct net_statistics stats_total;

int net_set_nonblocking(int fd, int toggle)
{
	int ret;
	uhub_log(log_trace, "net_set_nonblocking(): fd=%d", fd);
	ret = ioctl(fd, FIONBIO, &toggle);
    if (ret == -1)
	{
		uhub_log(log_error, "net_set_nonblocking(): ioctl failed (fd=%d): %s", fd, strerror(errno));
		return -1;
	}
	return 0;
}


int net_set_reuseaddress(int fd, int toggle)
{
	int ret;
	uhub_log(log_trace, "net_set_reuseaddress(): fd=%d", fd);
	ret = setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, &toggle, sizeof(toggle));
	if (ret == -1)
	{
		uhub_log(log_error, "net_set_reuseaddress(): setsockopt failed (fd=%d): %s", fd, strerror(errno));
		return -1;
	}
	return 0;
}


int net_close(int fd)
{
	int ret = close(fd);
	if (ret == 0)
	{
		net_stats_add_close();
	}
	else
	{
		if (fd != -1)
		{
			net_stats_add_error();
		}
	}
	return ret;
}


int net_accept(int fd)
{
	struct sockaddr_in6 addr;
	socklen_t addr_size;
	int ret = 0;
	addr_size = sizeof(struct sockaddr_in6);
	memset(&addr, 0, addr_size);
	uhub_log(log_trace, "net_accept() fd=%d", fd);
	ret = accept(fd, (struct sockaddr*) &addr, &addr_size);
	
	if (ret == -1)
	{
		switch (errno)
		{
#if defined(__linux__)
			case ENETDOWN:
			case EPROTO:
			case ENOPROTOOPT:
			case EHOSTDOWN:
			case ENONET:
			case EHOSTUNREACH:
			case EOPNOTSUPP:
				errno = EAGAIN;
#endif
			case EAGAIN:
				break;
			default:
				uhub_log(log_error, "net_accept(): accept failed (fd=%d, errno=%d, msg=%s)", fd, errno, strerror(errno));
				net_stats_add_error();
				return -1;
		}
	}
	else
	{
		net_stats_add_accept();
	}
	
	return ret;
}



int net_is_ipv6_supported()
{
	if (is_ipv6_supported == -1)
	{
		int ret = socket(AF_INET6, SOCK_DGRAM, IPPROTO_UDP);
		if (ret == -1)
		{
			if (errno == EAFNOSUPPORT)
			{
				is_ipv6_supported = 0;
				return 0;
			}
		
		}
		else
		{
			close(ret);
		}
		is_ipv6_supported = 1;
	}
	return is_ipv6_supported;
}


int net_socket_create(int af, int type, int protocol)
{
	int sd = socket(af, type, protocol);
	if (sd == -1)
	{
		uhub_log(log_error, "net_socket_create(): socket failed (errno=%d, msg=%s)", errno, strerror(errno));
	}
	return sd;
}


const char* net_get_peer_address(int fd)
{
	static char address[INET6_ADDRSTRLEN+1];
	struct sockaddr_in6 name;
	socklen_t namelen = sizeof(struct sockaddr_in6);
	int af = net_is_ipv6_supported() ? AF_INET6 : AF_INET;
	
	memset(address, 0, INET6_ADDRSTRLEN);
	
	if (getpeername(fd, (struct sockaddr*) &name, &namelen) != -1)
	{
		inet_ntop(af, (void*) &(&name)->sin6_addr, address, INET6_ADDRSTRLEN);
		if (strncmp(address, "::ffff:", 7) == 0) /* IPv6 mapped IPv4 address. */
		{
			return &address[7];
		}
		uhub_log(log_trace, "net_get_peer_address(): address=%s", address);
		return address;
	}
	else
	{
		uhub_log(log_error, "net_get_peer_address(): getsockname failed (fd=%d, errno=%d, msg=%s)", fd, errno, strerror(errno));
	}
	
	return "0.0.0.0";
}


ssize_t net_recv(int fd, void* buf, size_t len, int flags)
{
	ssize_t ret = recv(fd, buf, len, flags);
	if (ret >= 0)
	{
		net_stats_add_rx(ret);	
	}
	else
	{
		if (errno != EAGAIN)
			net_stats_add_error();
	}
	return ret;
}


ssize_t net_send(int fd, void* buf, size_t len, int flags)
{
	ssize_t ret = send(fd, buf, len, flags);
	if (ret >= 0)
	{
		net_stats_add_tx(ret);	
	}
	else
	{
		if (errno != EAGAIN)
			net_stats_add_error();
	}
	return ret;
}


void net_stats_initialize()
{
	memset(&stats_total, 0, sizeof(struct net_statistics));
	stats_total.timestamp = time(NULL);
	
	memset(&stats, 0, sizeof(struct net_statistics));
	stats.timestamp = time(NULL);
}


void net_stats_report()
{
	int factor = (time(NULL) - stats.timestamp);
	if (!factor) factor++;
	
	uhub_log(log_info, "Statistics  NET: tx=%d KB/s, rx=%d KB/s, (acc=%d/cls=%d/err=%d)",
		(int) ((stats.tx / factor) / 1024),
		(int) ((stats.rx / factor) / 1024),
		(int) stats.accept,
		(int) stats.closed,
		(int) stats.errors);
}


void net_stats_reset()
{
	stats_total.tx += stats.tx;
	stats_total.rx += stats.rx;
	stats_total.accept += stats.accept;
	stats_total.errors += stats.errors;
	stats_total.closed += stats.closed;
	
	memset(&stats, 0, sizeof(struct net_statistics));
	stats.timestamp = time(NULL);
}


int net_stats_timeout()
{
	/* FIXME: Configurable time for dumping network statistics */
	return (time(NULL) - stats.timestamp > 60) ? 1 : 0;
}


void net_stats_add_tx(size_t bytes)
{
	stats.tx += bytes;
}


void net_stats_add_rx(size_t bytes)
{
	stats.rx += bytes;
}


void net_stats_add_accept()
{
	stats.accept++;
}


void net_stats_add_error()
{
	stats.errors++;
}


void net_stats_add_close()
{
	stats.closed++;
}
